#include "rca.h"

#define SSID       ""
#define PASSWORD   ""

#define AP_SSID       "M0_AP"
#define AP_PASSWORD   "none"

#define WMODE_STATION

#include "ESP8266.h"

ESP8266 wifi;

void setup()
{
    int ret;
    
    Serial.println("Web Server Task : Start!");
    
    ret = wifi.begin(Serial5, 115200);

    if(ret == 1) {
        Serial.print("Cannot communicate with ESP8266.");
        while(1);        
    } else if(ret == 2) {
        Serial.println("FW Version mismatch.");
        Serial.print("FW Version:");
        Serial.println(wifi.getVersion().c_str());
        Serial.print("Supported FW Version:");
        Serial.println(ESP8266_SUPPORT_VERSION);
        while(1);
    } else {
        Serial.print("begin ok\r\n");
    }

    Serial.print("FW Version:");
    Serial.println(wifi.getVersion().c_str());
    
#ifdef  WMODE_STATION
    if (wifi.setOprToStation()) {
        Serial.print("to station ok\r\n");
    } else {
        Serial.print("to station err\r\n");
    }
    
    if (wifi.joinAP(SSID, PASSWORD)) {
        Serial.print("Join AP success\r\n");
        Serial.print("IP: ");
        Serial.println(wifi.getLocalIP().c_str());    
    } else {
        Serial.print("Join AP failure\r\n");
    }
#else /* !WMODE_STATION */ 
    if (wifi.setOprToSoftAP()) {
        Serial.print("to softap ok\r\n");
    } else {
        Serial.print("to softap err\r\n");
    }    
    if(wifi.setSoftAPParam(AP_SSID, AP_PASSWORD, 7, 0)){
        Serial.print("Set SoftAP success\r\n");
        Serial.print("IP: ");
        Serial.println(wifi.getLocalIP().c_str());            
    }
    else {
        Serial.print("Set SoftAP failure\r\n");
    }
#endif /* WMODE_STATION */
    
    if (wifi.enableMUX()) {
        Serial.print("multiple ok\r\n");
    } else {
        Serial.print("multiple err\r\n");
    }

    if (wifi.startTCPServer(80)) {
        Serial.print("start tcp server ok\r\n");
    } else {
        Serial.print("start tcp server err\r\n");
    }

    if (wifi.setTCPServerTimeout(10)) { 
        Serial.print("set tcp server timout 10 seconds\r\n");
    } else {
        Serial.print("set tcp server timout err\r\n");
    }

    Serial.print("setup end\r\n");
}

uint8_t mux_id_ptn;

void loop()
{
    uint8_t mux_id;
    uint8_t buffer[128] = {0};
    uint8_t pre_mux_id_ptn;
    uint32_t len
      
    pre_mux_id_ptn = mux_id_ptn;

    if(!wifi.getMuxCStatus(&mux_id_ptn)) {
        Serial.println("isConnected(&mux_id_ptn) : Error!");
        while(1);
    }
    
    if (pre_mux_id_ptn != mux_id_ptn) {
        Serial.print("Connection Status changed! : 0x");
        Serial.println(mux_id_ptn, HEX);
    }

    if(!wifi.isDataAvailable()) {
        return;
    }
      
    if((len = wifi.recv(&mux_id, buffer, sizeof(buffer))) == 0) {
        return;
    }

    String cmd = (char*)buffer;
    
    Serial.print("Web Server Task : Recive Request from mux : ");
    Serial.println(mux_id);
    Serial.print("Web Server Task : Recive len   : ");
    Serial.println(len);
    Serial.print("Web Server Task : Recive Data  : ");
    Serial.println(cmd);    
    
    if(!wifi.send(mux_id, cmd)) {
        Serial.println("send(mux_id, cmd) : Error!");
        while(1);
    }
    
    wifi.releaseTCP(mux_id);
}
